@@ -3,7 +3,7 @@
 from typing import Iterable, List, Optional, Sequence
 
 from mypy.types import (
-    CallableType, Type, TypeVisitor, UnboundType, AnyType, NoneTyp, TypeVarType, Instance,
+    CallableType, Type, TypeVisitor, UnboundType, AnyType, NoneType, TypeVarType, Instance,
     TupleType, TypedDictType, UnionType, Overloaded, ErasedType, PartialType, DeletedType,
     UninhabitedType, TypeType, TypeVarId, TypeQuery, is_named_instance, TypeOfAny, LiteralType,
 )
@@ -248,7 +248,7 @@ class ConstraintBuilderVisitor(TypeVisitor[List[Constraint]]):
     def visit_any(self, template: AnyType) -> List[Constraint]:
         return []
 
-    def visit_none_type(self, template: NoneTyp) -> List[Constraint]:
+    def visit_none_type(self, template: NoneType) -> List[Constraint]:
         return []
 
     def visit_uninhabited_type(self, template: UninhabitedType) -> List[Constraint]:
