@@ -14,8 +14,9 @@ import subprocess
 import sys
 import time
 import traceback
+from contextlib import redirect_stderr, redirect_stdout
 
-from typing import AbstractSet, Any, Callable, Dict, List, Mapping, Optional, Sequence, Set, Tuple
+from typing import AbstractSet, Any, Callable, Dict, List, Mapping, Optional, Sequence, Tuple
 
 import mypy.build
 import mypy.errors
@@ -28,8 +29,8 @@ from mypy.fscache import FileSystemCache
 from mypy.fswatcher import FileSystemWatcher, FileData
 from mypy.modulefinder import BuildSource, compute_search_paths
 from mypy.options import Options
+from mypy.suggestions import SuggestionFailure, SuggestionEngine
 from mypy.typestate import reset_global_state
-from mypy.util import redirect_stderr, redirect_stdout
 from mypy.version import __version__
 
 
@@ -262,10 +263,16 @@ class Server:
 
     # Command functions (run in the server via RPC).
 
-    def cmd_status(self) -> Dict[str, object]:
+    def cmd_status(self, fswatcher_dump_file: Optional[str] = None) -> Dict[str, object]:
         """Return daemon status."""
         res = {}  # type: Dict[str, object]
         res.update(get_meminfo())
+        if fswatcher_dump_file:
+            data = self.fswatcher.dump_file_data() if hasattr(self, 'fswatcher') else {}
+            # Using .dumps and then writing was noticably faster than using dump
+            s = json.dumps(data)
+            with open(fswatcher_dump_file, 'w') as f:
+                f.write(s)
         return res
 
     def cmd_stop(self) -> Dict[str, object]:
@@ -301,7 +308,9 @@ class Server:
             if self.fine_grained_manager:
                 manager = self.fine_grained_manager.manager
                 start_plugins_snapshot = manager.plugins_snapshot
-                _, current_plugins_snapshot = mypy.build.load_plugins(options, manager.errors)
+                _, current_plugins_snapshot = mypy.build.load_plugins(options,
+                                                                      manager.errors,
+                                                                      sys.stdout)
                 if current_plugins_snapshot != start_plugins_snapshot:
                     return {'restart': 'plugins changed'}
         except InvalidSourceList as err:
@@ -507,6 +516,30 @@ class Server:
 
         return changed, removed
 
+    def cmd_suggest(self,
+                    function: str,
+                    callsites: bool,
+                    **kwargs: bool) -> Dict[str, object]:
+        """Suggest a signature for a function."""
+        if not self.fine_grained_manager:
+            return {'error': "Command 'suggest' is only valid after a 'check' command"}
+        engine = SuggestionEngine(self.fine_grained_manager, **kwargs)
+        try:
+            if callsites:
+                out = engine.suggest_callsites(function)
+            else:
+                out = engine.suggest(function)
+        except SuggestionFailure as err:
+            return {'error': str(err)}
+        else:
+            if not out:
+                out = "No suggestions\n"
+            elif not out.endswith("\n"):
+                out += "\n"
+            return {'out': out, 'err': "", 'status': 0}
+        finally:
+            self.fscache.flush()
+
     def cmd_hang(self) -> Dict[str, object]:
         """Hang for 100 seconds, as a debug hack."""
         time.sleep(100)
@@ -524,13 +557,12 @@ def get_meminfo() -> Dict[str, Any]:
     try:
         import psutil  # type: ignore  # It's not in typeshed yet
     except ImportError:
-        if sys.platform != 'win32':
-            res['memory_psutil_missing'] = (
-                'psutil not found, run pip install mypy[dmypy] '
-                'to install the needed components for dmypy'
-            )
+        res['memory_psutil_missing'] = (
+            'psutil not found, run pip install mypy[dmypy] '
+            'to install the needed components for dmypy'
+        )
     else:
-        process = psutil.Process(os.getpid())
+        process = psutil.Process()
         meminfo = process.memory_info()
         res['memory_rss_mib'] = meminfo.rss / MiB
         res['memory_vms_mib'] = meminfo.vms / MiB
@@ -540,9 +572,7 @@ def get_meminfo() -> Dict[str, Any]:
             # See https://stackoverflow.com/questions/938733/total-memory-used-by-python-process
             import resource  # Since it doesn't exist on Windows.
             rusage = resource.getrusage(resource.RUSAGE_SELF)
-            # mypyc doesn't like unreachable code, so trick mypy into thinking
-            # the branch is reachable
-            if sys.platform == 'darwin' or bool():
+            if sys.platform == 'darwin':
                 factor = 1
             else:
                 factor = 1024  # Linux
