@@ -3,7 +3,7 @@ import sys
 import traceback
 from collections import OrderedDict, defaultdict
 
-from typing import Tuple, List, TypeVar, Set, Dict, Optional
+from typing import Tuple, List, TypeVar, Set, Dict, Optional, TextIO
 
 from mypy.scope import Scope
 from mypy.options import Options
@@ -42,7 +42,7 @@ class ErrorInfo:
     # The column number related to this error with file.
     column = 0   # -1 if unknown
 
-    # Either 'error', 'note', or 'warning'.
+    # Either 'error' or 'note'
     severity = ''
 
     # The error message.
@@ -54,8 +54,9 @@ class ErrorInfo:
     # Only report this particular messages once per program.
     only_once = False
 
-    # Actual origin of the error message as tuple (path, line number)
-    origin = None  # type: Tuple[str, int]
+    # Actual origin of the error message as tuple (path, line number, end line number)
+    # If end line number is unknown, use line number.
+    origin = None  # type: Tuple[str, int, int]
 
     # Fine-grained incremental target where this was reported
     target = None  # type: Optional[str]
@@ -72,7 +73,7 @@ class ErrorInfo:
                  message: str,
                  blocker: bool,
                  only_once: bool,
-                 origin: Optional[Tuple[str, int]] = None,
+                 origin: Optional[Tuple[str, int, int]] = None,
                  target: Optional[str] = None) -> None:
         self.import_ctx = import_ctx
         self.file = file
@@ -85,7 +86,7 @@ class ErrorInfo:
         self.message = message
         self.blocker = blocker
         self.only_once = only_once
-        self.origin = origin or (file, line)
+        self.origin = origin or (file, line, line)
         self.target = target
 
 
@@ -169,6 +170,9 @@ class Errors:
         new.scope = self.scope
         return new
 
+    def total_errors(self) -> int:
+        return sum(len(errs) for errs in self.error_info_map.values())
+
     def set_ignore_prefix(self, prefix: str) -> None:
         """Set path prefix that will be removed from all paths."""
         prefix = os.path.normpath(prefix)
@@ -230,17 +234,19 @@ class Errors:
                file: Optional[str] = None,
                only_once: bool = False,
                origin_line: Optional[int] = None,
-               offset: int = 0) -> None:
+               offset: int = 0,
+               end_line: Optional[int] = None) -> None:
         """Report message at the given line using the current error context.
 
         Args:
             line: line number of error
             message: message to report
             blocker: if True, don't continue analysis after this error
-            severity: 'error', 'note' or 'warning'
+            severity: 'error' or 'note'
             file: if non-None, override current file as context
             only_once: if True, only report this exact message once per build
             origin_line: if non-None, override current context as origin
+            end_line: if non-None, override current context as end
         """
         if self.scope:
             type = self.scope.current_type_name()
@@ -257,10 +263,17 @@ class Errors:
             file = self.file
         if offset:
             message = " " * offset + message
+
+        if origin_line is None:
+            origin_line = line
+
+        if end_line is None:
+            end_line = origin_line
+
         info = ErrorInfo(self.import_context(), file, self.current_module(), type,
                          function, line, column, severity, message,
                          blocker, only_once,
-                         origin=(self.file, origin_line) if origin_line else None,
+                         origin=(self.file, origin_line, end_line),
                          target=self.current_target())
         self.add_error_info(info)
 
@@ -271,12 +284,22 @@ class Errors:
         self.error_info_map[file].append(info)
 
     def add_error_info(self, info: ErrorInfo) -> None:
-        file, line = info.origin
+        file, line, end_line = info.origin
         if not info.blocker:  # Blockers cannot be ignored
-            if file in self.ignored_lines and line in self.ignored_lines[file]:
-                # Annotation requests us to ignore all errors on this line.
-                self.used_ignored_lines[file].add(line)
-                return
+            if file in self.ignored_lines:
+                # It's okay if end_line is *before* line.
+                # Function definitions do this, for example, because the correct
+                # error reporting line is at the *end* of the ignorable range
+                # (for compatibility reasons). If so, just flip 'em!
+                if end_line < line:
+                    line, end_line = end_line, line
+                # Check each line in this context for "type: ignore" comments.
+                # line == end_line for most nodes, so we only loop once.
+                for scope_line in range(line, end_line + 1):
+                    if scope_line in self.ignored_lines[file]:
+                        # Annotation requests us to ignore all errors on this line.
+                        self.used_ignored_lines[file].add(scope_line)
+                        return
             if file in self.ignored_files:
                 return
         if info.only_once:
@@ -296,13 +319,13 @@ class Errors:
                     self.only_once_messages.remove(info.message)
             self.error_info_map[path] = new_errors
 
-    def generate_unused_ignore_notes(self, file: str) -> None:
+    def generate_unused_ignore_errors(self, file: str) -> None:
         ignored_lines = self.ignored_lines[file]
         if not self.is_typeshed_file(file) and file not in self.ignored_files:
             for line in ignored_lines - self.used_ignored_lines[file]:
                 # Don't use report since add_error_info will ignore the error!
                 info = ErrorInfo(self.import_context(), file, self.current_module(), None,
-                                 None, line, -1, 'note', "unused 'type: ignore' comment",
+                                 None, line, -1, 'error', "unused 'type: ignore' comment",
                                  False, False)
                 self._add_error_info(file, info)
 
@@ -562,19 +585,27 @@ def remove_path_prefix(path: str, prefix: Optional[str]) -> str:
         return path
 
 
-def report_internal_error(err: Exception, file: Optional[str], line: int,
-                          errors: Errors, options: Options) -> None:
+def report_internal_error(err: Exception,
+                          file: Optional[str],
+                          line: int,
+                          errors: Errors,
+                          options: Options,
+                          stdout: Optional[TextIO] = None,
+                          stderr: Optional[TextIO] = None,
+                          ) -> None:
     """Report internal error and exit.
 
     This optionally starts pdb or shows a traceback.
     """
+    stdout = (stdout or sys.stdout)
+    stderr = (stderr or sys.stderr)
     # Dump out errors so far, they often provide a clue.
     # But catch unexpected errors rendering them.
     try:
         for msg in errors.new_messages():
             print(msg)
     except Exception as e:
-        print("Failed to dump errors:", repr(e), file=sys.stderr)
+        print("Failed to dump errors:", repr(e), file=stderr)
 
     # Compute file:line prefix for official-looking error messages.
     if file:
@@ -587,13 +618,22 @@ def report_internal_error(err: Exception, file: Optional[str], line: int,
 
     # Print "INTERNAL ERROR" message.
     print('{}error: INTERNAL ERROR --'.format(prefix),
-          'please report a bug at https://github.com/python/mypy/issues',
-          'version: {}'.format(mypy_version),
-          file=sys.stderr)
+          'Please try using mypy master on Github:\n'
+          'https://mypy.rtfd.io/en/latest/common_issues.html#using-development-mypy-build',
+          file=stderr)
+    if options.show_traceback:
+        print('Please report a bug at https://github.com/python/mypy/issues',
+            file=stderr)
+    else:
+        print('If this issue continues with mypy master, '
+              'please report a bug at https://github.com/python/mypy/issues',
+            file=stderr)
+    print('version: {}'.format(mypy_version),
+          file=stderr)
 
     # If requested, drop into pdb. This overrides show_tb.
     if options.pdb:
-        print('Dropping into pdb', file=sys.stderr)
+        print('Dropping into pdb', file=stderr)
         import pdb
         pdb.post_mortem(sys.exc_info()[2])
 
@@ -604,15 +644,15 @@ def report_internal_error(err: Exception, file: Optional[str], line: int,
         if not options.pdb:
             print('{}: note: please use --show-traceback to print a traceback '
                   'when reporting a bug'.format(prefix),
-                  file=sys.stderr)
+                  file=stderr)
     else:
         tb = traceback.extract_stack()[:-2]
         tb2 = traceback.extract_tb(sys.exc_info()[2])
         print('Traceback (most recent call last):')
         for s in traceback.format_list(tb + tb2):
             print(s.rstrip('\n'))
-        print('{}: {}'.format(type(err).__name__, err))
-        print('{}: note: use --pdb to drop into pdb'.format(prefix), file=sys.stderr)
+        print('{}: {}'.format(type(err).__name__, err), file=stdout)
+        print('{}: note: use --pdb to drop into pdb'.format(prefix), file=stderr)
 
     # Exit.  The caller has nothing more to say.
     # We use exit code 2 to signal that this is no ordinary error.
