@@ -1,6 +1,6 @@
 """Hack for handling non-mypyc compiled plugins with a mypyc-compiled mypy"""
 
-from typing import Optional, Callable, Any, Dict
+from typing import Optional, Callable, Any, Dict, List, Tuple
 from mypy.options import Options
 from mypy.types import Type, CallableType
 from mypy.nodes import SymbolTableNode, MypyFile
@@ -23,7 +23,7 @@ class InterpretedPlugin:
 
     def __new__(cls, *args: Any, **kwargs: Any) -> 'mypy.plugin.Plugin':
         from mypy.plugin import WrapperPlugin
-        plugin = object.__new__(cls)  # type: ignore
+        plugin = object.__new__(cls)
         plugin.__init__(*args, **kwargs)
         return WrapperPlugin(plugin)
 
@@ -39,6 +39,9 @@ class InterpretedPlugin:
         assert self._modules is not None
         return lookup_fully_qualified(fullname, self._modules)
 
+    def get_additional_deps(self, file: MypyFile) -> List[Tuple[int, str, int]]:
+        return []
+
     def get_type_analyze_hook(self, fullname: str
                               ) -> Optional[Callable[['mypy.plugin.AnalyzeTypeContext'], Type]]:
         return None
