@@ -39,8 +39,8 @@ Every value stored there must be JSON-serializable.
 
 import types
 
-from abc import abstractmethod
-from typing import Callable, List, Tuple, Optional, NamedTuple, TypeVar, Dict
+from abc import abstractmethod, abstractproperty
+from typing import Any, Callable, List, Tuple, Optional, NamedTuple, TypeVar, Dict
 from mypy_extensions import trait
 
 from mypy.nodes import (
@@ -131,6 +131,7 @@ class CheckerPluginInterface:
 
     msg = None  # type: MessageBuilder
     options = None  # type: Options
+    path = None  # type: str
 
     @abstractmethod
     def fail(self, msg: str, ctx: Context) -> None:
@@ -244,7 +245,7 @@ class SemanticAnalyzerPluginInterface:
         raise NotImplementedError
 
     @abstractmethod
-    def add_symbol_table_node(self, name: str, stnode: SymbolTableNode) -> None:
+    def add_symbol_table_node(self, name: str, stnode: SymbolTableNode) -> Any:
         """Add node to global symbol table (or to nearest class if there is one)."""
         raise NotImplementedError
 
@@ -253,6 +254,23 @@ class SemanticAnalyzerPluginInterface:
         """Make qualified name using current module and enclosing class (if any)."""
         raise NotImplementedError
 
+    @abstractmethod
+    def defer(self) -> None:
+        """Call this to defer the processing of the current node.
+
+        This will request an additional iteration of semantic analysis.
+        Only available with new semantic analyzer.
+        """
+        raise NotImplementedError
+
+    @abstractproperty
+    def final_iteration(self) -> bool:
+        """Is this the final iteration of semantic analysis?
+
+        Only available with new semantic analyzer.
+        """
+        raise NotImplementedError
+
 
 # A context for a function hook that infers the return type of a function with
 # a special signature.
@@ -361,6 +379,23 @@ class Plugin(CommonPluginApi):
         assert self._modules is not None
         return lookup_fully_qualified(fullname, self._modules)
 
+    def get_additional_deps(self, file: MypyFile) -> List[Tuple[int, str, int]]:
+        """Customize dependencies for a module.
+
+        This hook allows adding in new dependencies for a module. It
+        is called after parsing a file but before analysis. This can
+        be useful if a library has dependencies that are dynamic based
+        on configuration information, for example.
+
+        Returns a list of (priority, module name, line number) tuples.
+
+        The line number can be -1 when there is not a known real line number.
+
+        Priorities are defined in mypy.build (but maybe shouldn't be).
+        10 is a good choice for priority.
+        """
+        return []
+
     def get_type_analyze_hook(self, fullname: str
                               ) -> Optional[Callable[[AnalyzeTypeContext], Type]]:
         """Customize behaviour of the type analyzer for given full names.
@@ -385,7 +420,7 @@ class Plugin(CommonPluginApi):
         """Adjust the return type of a function call.
 
         This method is called after type checking a call. Plugin may adjust the return
-        type inferred by mypy, and/or emmit some error messages. Note, this hook is also
+        type inferred by mypy, and/or emit some error messages. Note, this hook is also
         called for class instantiation calls, so that in this example:
 
             from lib import Class, do_stuff
@@ -440,8 +475,15 @@ class Plugin(CommonPluginApi):
         """Adjust type of a class attribute.
 
         This method is called with attribute full name using the class where the attribute was
-        defined (or Var.info.fullname() for generated attributes). Currently, this hook is only
-        called for names that exist in the class MRO, for example in:
+        defined (or Var.info.fullname() for generated attributes).
+
+        For classes without __getattr__ or __getattribute__, this hook is only called for
+        names of fields/properties (but not methods) that exist in the instance MRO.
+
+        For classes that implement __getattr__ or __getattribute__, this hook is called
+        for all fields/properties, including nonexistent ones (but still not methods).
+
+        For example:
 
             class Base:
                 x: Any
@@ -454,7 +496,9 @@ class Plugin(CommonPluginApi):
             var.x
             var.y
 
-        this method is only called with '__main__.Base.x'.
+        get_attribute_hook is called with '__main__.Base.x' and '__main__.Base.y'.
+        However, if we had not implemented __getattr__ on Base, you would only get
+        the callback for 'var.x'; 'var.y' would produce an error without calling the hook.
         """
         return None
 
@@ -542,6 +586,9 @@ class WrapperPlugin(Plugin):
     def lookup_fully_qualified(self, fullname: str) -> Optional[SymbolTableNode]:
         return self.plugin.lookup_fully_qualified(fullname)
 
+    def get_additional_deps(self, file: MypyFile) -> List[Tuple[int, str, int]]:
+        return self.plugin.get_additional_deps(file)
+
     def get_type_analyze_hook(self, fullname: str
                               ) -> Optional[Callable[[AnalyzeTypeContext], Type]]:
         return self.plugin.get_type_analyze_hook(fullname)
@@ -607,6 +654,12 @@ class ChainedPlugin(Plugin):
         for plugin in self._plugins:
             plugin.set_modules(modules)
 
+    def get_additional_deps(self, file: MypyFile) -> List[Tuple[int, str, int]]:
+        deps = []
+        for plugin in self._plugins:
+            deps.extend(plugin.get_additional_deps(file))
+        return deps
+
     def get_type_analyze_hook(self, fullname: str
                               ) -> Optional[Callable[[AnalyzeTypeContext], Type]]:
         return self._find_hook(lambda plugin: plugin.get_type_analyze_hook(fullname))
