@@ -114,8 +114,7 @@ test cases (test-data/unit/fine-grained*.test).
 
 import time
 from typing import (
-    Dict, List, Set, Tuple, Iterable, Union, Optional, NamedTuple, Callable,
-    Sequence
+    Dict, List, Set, Tuple, Union, Optional, NamedTuple, Callable, Sequence
 )
 
 from mypy.build import (
@@ -136,11 +135,14 @@ from mypy.semanal import apply_semantic_analyzer_patches
 from mypy.server.astdiff import (
     snapshot_symbol_table, compare_symbol_table_snapshots, SnapshotItem
 )
+from mypy.newsemanal.semanal_main import semantic_analysis_for_scc, semantic_analysis_for_targets
 from mypy.server.astmerge import merge_asts
 from mypy.server.aststrip import strip_target
+from mypy.server.aststripnew import strip_target_new, SavedAttributes
 from mypy.server.deps import get_dependencies_of_target
-from mypy.server.target import module_prefix, split_target, trigger_to_target
+from mypy.server.target import trigger_to_target
 from mypy.server.trigger import make_trigger, WILDCARD_TAG
+from mypy.util import module_prefix, split_target
 from mypy.typestate import TypeState
 
 MYPY = False
@@ -183,6 +185,8 @@ class FineGrainedBuildManager:
         self.changed_modules = []  # type: List[Tuple[str, str]]
         # Modules processed during the last update
         self.updated_modules = []  # type: List[str]
+        # Targets processed during last update (for testing only).
+        self.processed_targets = []  # type: List[str]
 
     def update(self,
                changed_modules: List[Tuple[str, str]],
@@ -205,6 +209,7 @@ class FineGrainedBuildManager:
         Returns:
             A list of errors.
         """
+        self.processed_targets.clear()
         changed_modules = changed_modules + removed_modules
         removed_set = {module for module, _ in removed_modules}
         self.changed_modules = changed_modules
@@ -254,7 +259,7 @@ class FineGrainedBuildManager:
                 # which prevents us from reprocessing errors in it.
                 changed_modules = propagate_changes_using_dependencies(
                     self.manager, self.graph, self.deps, set(), {next_id},
-                    self.previous_targets_with_errors)
+                    self.previous_targets_with_errors, self.processed_targets)
                 changed_modules = dedupe_modules(changed_modules)
                 if not changed_modules:
                     # Preserve state needed for the next update.
@@ -265,6 +270,20 @@ class FineGrainedBuildManager:
         self.previous_messages = messages[:]
         return messages
 
+    def trigger(self, target: str) -> List[str]:
+        """Trigger a specific target explicitly.
+
+        This is intended for use by the suggestions engine.
+        """
+        self.manager.errors.reset()
+        changed_modules = propagate_changes_using_dependencies(
+            self.manager, self.graph, self.deps, set(), set(),
+            self.previous_targets_with_errors | {target}, [])
+        # Preserve state needed for the next update.
+        self.previous_targets_with_errors = self.manager.errors.targets()
+        self.previous_messages = self.manager.errors.new_messages()[:]
+        return self.update(changed_modules, [])
+
     def update_one(self,
                    changed_modules: List[Tuple[str, str]],
                    initial_set: Set[str],
@@ -343,6 +362,7 @@ class FineGrainedBuildManager:
             old_snapshots[module] = snapshot
 
         manager.errors.reset()
+        self.processed_targets.append(module)
         result = update_module_isolated(module, path, manager, previous_modules, graph,
                                         force_removed)
         if isinstance(result, BlockedUpdate):
@@ -366,7 +386,7 @@ class FineGrainedBuildManager:
         remaining += propagate_changes_using_dependencies(
             manager, graph, self.deps, triggered,
             {module},
-            targets_with_errors=set())
+            targets_with_errors=set(), processed_targets=self.processed_targets)
         t2 = time.time()
         manager.add_stats(
             update_isolated_time=t1 - t0,
@@ -562,20 +582,25 @@ def update_module_isolated(module: str,
     assert state.tree is not None, "file must be at least parsed"
     t0 = time.time()
     # TODO: state.fix_suppressed_dependencies()?
-    if module == 'typing':
-        # We need to manually add typing aliases to builtins, like we
-        # do in process_stale_scc. Because this can't be done until
-        # builtins is also loaded, there isn't an obvious way to
-        # refactor this.
-        manager.semantic_analyzer.add_builtin_aliases(state.tree)
+    if not manager.options.new_semantic_analyzer:
+        if module == 'typing':
+            # We need to manually add typing aliases to builtins, like we
+            # do in process_stale_scc. Because this can't be done until
+            # builtins is also loaded, there isn't an obvious way to
+            # refactor this.
+            manager.semantic_analyzer.add_builtin_aliases(state.tree)
     try:
-        state.semantic_analysis()
+        if not manager.options.new_semantic_analyzer:
+            state.semantic_analysis()
+        else:
+            semantic_analysis_for_scc(graph, [state.id], manager.errors)
     except CompileError as err:
         # There was a blocking error, so module AST is incomplete. Restore old modules.
         restore([module])
         return BlockedUpdate(module, path, remaining_modules, err.messages)
-    state.semantic_analysis_pass_three()
-    state.semantic_analysis_apply_patches()
+    if not manager.options.new_semantic_analyzer:
+        state.semantic_analysis_pass_three()
+        state.semantic_analysis_apply_patches()
 
     # Merge old and new ASTs.
     new_modules_dict = {module: state.tree}  # type: Dict[str, Optional[MypyFile]]
@@ -762,11 +787,16 @@ def propagate_changes_using_dependencies(
         deps: Dict[str, Set[str]],
         triggered: Set[str],
         up_to_date_modules: Set[str],
-        targets_with_errors: Set[str]) -> List[Tuple[str, str]]:
+        targets_with_errors: Set[str],
+        processed_targets: List[str]) -> List[Tuple[str, str]]:
     """Transitively rechecks targets based on triggers and the dependency map.
 
     Returns a list (module id, path) tuples representing modules that contain
-    a target that needs to be reprocessed but that has not been parsed yet."""
+    a target that needs to be reprocessed but that has not been parsed yet.
+
+    Processed targets should be appended to processed_targets (used in tests only,
+    to test the order of processing targets).
+    """
 
     num_iter = 0
     remaining_modules = []  # type: List[Tuple[str, str]]
@@ -802,7 +832,7 @@ def propagate_changes_using_dependencies(
         # TODO: Preserve order (set is not optimal)
         for id, nodes in sorted(todo.items(), key=lambda x: x[0]):
             assert id not in up_to_date_modules
-            triggered |= reprocess_nodes(manager, graph, id, nodes, deps)
+            triggered |= reprocess_nodes(manager, graph, id, nodes, deps, processed_targets)
         # Changes elsewhere may require us to reprocess modules that were
         # previously considered up to date. For example, there may be a
         # dependency loop that loops back to an originally processed module.
@@ -878,7 +908,8 @@ def reprocess_nodes(manager: BuildManager,
                     graph: Dict[str, State],
                     module_id: str,
                     nodeset: Set[FineGrainedDeferredNode],
-                    deps: Dict[str, Set[str]]) -> Set[str]:
+                    deps: Dict[str, Set[str]],
+                    processed_targets: List[str]) -> Set[str]:
     """Reprocess a set of nodes within a single module.
 
     Return fired triggers.
@@ -912,11 +943,71 @@ def reprocess_nodes(manager: BuildManager,
             targets.add(target)
     manager.errors.clear_errors_in_targets(file_node.path, targets)
 
-    # Strip semantic analysis information.
-    for deferred in nodes:
-        strip_target(deferred.node)
-    semantic_analyzer = manager.semantic_analyzer
+    # If one of the nodes is the module itself, emit any errors that
+    # happened before semantic analysis.
+    for target in targets:
+        if target == module_id:
+            for info in graph[module_id].early_errors:
+                manager.errors.add_error_info(info)
 
+    # Strip semantic analysis information.
+    saved_attrs = {}  # type: SavedAttributes
+    for deferred in nodes:
+        processed_targets.append(deferred.node.fullname())
+        if not manager.options.new_semantic_analyzer:
+            strip_target(deferred.node)
+        else:
+            strip_target_new(deferred.node, saved_attrs)
+    if not options.new_semantic_analyzer:
+        re_analyze_nodes(file_node, nodes, manager, options)
+    else:
+        semantic_analysis_for_targets(graph[module_id], nodes, graph, saved_attrs)
+    # Merge symbol tables to preserve identities of AST nodes. The file node will remain
+    # the same, but other nodes may have been recreated with different identities, such as
+    # NamedTuples defined using assignment statements.
+    new_symbols = find_symbol_tables_recursive(file_node.fullname(), file_node.names)
+    for name in old_symbols:
+        if name in new_symbols:
+            merge_asts(file_node, old_symbols[name], file_node, new_symbols[name])
+
+    # Type check.
+    checker = graph[module_id].type_checker()
+    checker.reset()
+    # We seem to need additional passes in fine-grained incremental mode.
+    checker.pass_num = 0
+    checker.last_pass = 3
+    more = checker.check_second_pass(nodes)
+    while more:
+        more = False
+        if graph[module_id].type_checker().check_second_pass():
+            more = True
+
+    if manager.options.export_types:
+        manager.all_types.update(graph[module_id].type_map())
+
+    new_symbols_snapshot = snapshot_symbol_table(file_node.fullname(), file_node.names)
+    # Check if any attribute types were changed and need to be propagated further.
+    changed = compare_symbol_table_snapshots(file_node.fullname(),
+                                             old_symbols_snapshot,
+                                             new_symbols_snapshot)
+    new_triggered = {make_trigger(name) for name in changed}
+
+    # Dependencies may have changed.
+    update_deps(module_id, nodes, graph, deps, options)
+
+    # Report missing imports.
+    graph[module_id].verify_dependencies()
+
+    return new_triggered
+
+
+def re_analyze_nodes(file_node: MypyFile, nodes: List[FineGrainedDeferredNode],
+                     manager: BuildManager, options: Options) -> None:
+    """Perform semantic analysis of targets in a given file.
+
+    This uses the old semantic analyzer.
+    """
+    semantic_analyzer = manager.semantic_analyzer
     patches = []  # type: List[Tuple[int, Callable[[], None]]]
 
     # Second pass of semantic analysis. We don't redo the first pass, because it only
@@ -946,41 +1037,6 @@ def reprocess_nodes(manager: BuildManager,
             active_type=None):
         apply_semantic_analyzer_patches(patches)
 
-    # Merge symbol tables to preserve identities of AST nodes. The file node will remain
-    # the same, but other nodes may have been recreated with different identities, such as
-    # NamedTuples defined using assignment statements.
-    new_symbols = find_symbol_tables_recursive(file_node.fullname(), file_node.names)
-    for name in old_symbols:
-        if name in new_symbols:
-            merge_asts(file_node, old_symbols[name], file_node, new_symbols[name])
-
-    # Type check.
-    checker = graph[module_id].type_checker()
-    checker.reset()
-    # We seem to need additional passes in fine-grained incremental mode.
-    checker.pass_num = 0
-    checker.last_pass = 3
-    more = checker.check_second_pass(nodes)
-    while more:
-        more = False
-        if graph[module_id].type_checker().check_second_pass():
-            more = True
-
-    new_symbols_snapshot = snapshot_symbol_table(file_node.fullname(), file_node.names)
-    # Check if any attribute types were changed and need to be propagated further.
-    changed = compare_symbol_table_snapshots(file_node.fullname(),
-                                             old_symbols_snapshot,
-                                             new_symbols_snapshot)
-    new_triggered = {make_trigger(name) for name in changed}
-
-    # Dependencies may have changed.
-    update_deps(module_id, nodes, graph, deps, options)
-
-    # Report missing imports.
-    graph[module_id].verify_dependencies()
-
-    return new_triggered
-
 
 def find_symbol_tables_recursive(prefix: str, symbols: SymbolTable) -> Dict[str, SymbolTable]:
     """Find all nested symbol tables.
