@@ -2,27 +2,27 @@ from typing import List, Optional, Callable, Tuple, Iterator, Set, Union, cast
 from contextlib import contextmanager
 
 from mypy.types import (
-    Type, AnyType, UnboundType, TypeVisitor, FormalArgument, NoneTyp, function_type,
+    Type, AnyType, UnboundType, TypeVisitor, FormalArgument, NoneType, function_type,
     Instance, TypeVarType, CallableType, TupleType, TypedDictType, UnionType, Overloaded,
     ErasedType, PartialType, DeletedType, UninhabitedType, TypeType, is_named_instance,
     FunctionLike, TypeOfAny, LiteralType,
 )
 import mypy.applytype
 import mypy.constraints
+import mypy.typeops
+import mypy.sametypes
 from mypy.erasetype import erase_type
 # Circular import; done in the function instead.
 # import mypy.solve
-from mypy import messages, sametypes
+from mypy import messages
 from mypy.nodes import (
     FuncBase, Var, Decorator, OverloadedFuncDef, TypeInfo, CONTRAVARIANT, COVARIANT,
-    ARG_POS, ARG_OPT, ARG_NAMED, ARG_NAMED_OPT, ARG_STAR, ARG_STAR2
+    ARG_POS, ARG_OPT, ARG_STAR, ARG_STAR2
 )
 from mypy.maptype import map_instance_to_supertype
 from mypy.expandtype import expand_type_by_instance
-from mypy.sametypes import is_same_type
 from mypy.typestate import TypeState, SubtypeKind
-
-from mypy import experiments
+from mypy import state
 
 MYPY = False
 if MYPY:
@@ -164,9 +164,9 @@ class SubtypeVisitor(TypeVisitor[bool]):
     def visit_any(self, left: AnyType) -> bool:
         return True
 
-    def visit_none_type(self, left: NoneTyp) -> bool:
-        if experiments.STRICT_OPTIONAL:
-            return (isinstance(self.right, NoneTyp) or
+    def visit_none_type(self, left: NoneType) -> bool:
+        if state.strict_optional:
+            return (isinstance(self.right, NoneType) or
                     is_named_instance(self.right, 'builtins.object') or
                     isinstance(self.right, Instance) and self.right.type.is_protocol and
                     not self.right.type.protocol_members)
@@ -184,7 +184,7 @@ class SubtypeVisitor(TypeVisitor[bool]):
 
     def visit_instance(self, left: Instance) -> bool:
         if left.type.fallback_to_any:
-            if isinstance(self.right, NoneTyp):
+            if isinstance(self.right, NoneType):
                 # NOTE: `None` is a *non-subclassable* singleton, therefore no class
                 # can by a subtype of it, even with an `Any` fallback.
                 # This special case is needed to treat descriptors in classes with
@@ -192,8 +192,8 @@ class SubtypeVisitor(TypeVisitor[bool]):
                 return False
             return True
         right = self.right
-        if isinstance(right, TupleType) and right.fallback.type.is_enum:
-            return self._is_subtype(left, right.fallback)
+        if isinstance(right, TupleType) and mypy.typeops.tuple_fallback(right).type.is_enum:
+            return self._is_subtype(left, mypy.typeops.tuple_fallback(right))
         if isinstance(right, Instance):
             if TypeState.is_cached_subtype_check(self._subtype_kind, left, right):
                 return True
@@ -221,7 +221,7 @@ class SubtypeVisitor(TypeVisitor[bool]):
         if isinstance(right, TypeType):
             item = right.item
             if isinstance(item, TupleType):
-                item = item.fallback
+                item = mypy.typeops.tuple_fallback(item)
             if is_named_instance(left, 'builtins.type'):
                 return self._is_subtype(TypeType(AnyType(TypeOfAny.special_form)), right)
             if left.type.is_metaclass():
@@ -285,7 +285,7 @@ class SubtypeVisitor(TypeVisitor[bool]):
                 else:
                     iter_type = AnyType(TypeOfAny.special_form)
                 return all(self._is_subtype(li, iter_type) for li in left.items)
-            elif self._is_subtype(left.fallback, right):
+            elif self._is_subtype(mypy.typeops.tuple_fallback(left), right):
                 return True
             return False
         elif isinstance(right, TupleType):
@@ -294,7 +294,15 @@ class SubtypeVisitor(TypeVisitor[bool]):
             for l, r in zip(left.items, right.items):
                 if not self._is_subtype(l, r):
                     return False
-            if not self._is_subtype(left.fallback, right.fallback):
+            rfallback = mypy.typeops.tuple_fallback(right)
+            if is_named_instance(rfallback, 'builtins.tuple'):
+                # No need to verify fallback. This is useful since the calculated fallback
+                # may be inconsistent due to how we calculate joins between unions vs.
+                # non-unions. For example, join(int, str) == object, whereas
+                # join(Union[int, C], Union[str, C]) == Union[int, str, C].
+                return True
+            lfallback = mypy.typeops.tuple_fallback(left)
+            if not self._is_subtype(lfallback, rfallback):
                 return False
             return True
         else:
@@ -327,8 +335,11 @@ class SubtypeVisitor(TypeVisitor[bool]):
         else:
             return False
 
-    def visit_literal_type(self, t: LiteralType) -> bool:
-        raise NotImplementedError()
+    def visit_literal_type(self, left: LiteralType) -> bool:
+        if isinstance(self.right, LiteralType):
+            return left == self.right
+        else:
+            return self._is_subtype(left.fallback, self.right)
 
     def visit_overloaded(self, left: Overloaded) -> bool:
         right = self.right
@@ -456,7 +467,8 @@ def is_protocol_implementation(left: Instance, right: Instance,
     TypeState.record_protocol_subtype_check(left.type, right.type)
     assuming = right.type.assuming_proper if proper_subtype else right.type.assuming
     for (l, r) in reversed(assuming):
-        if sametypes.is_same_type(l, left) and sametypes.is_same_type(r, right):
+        if (mypy.sametypes.is_same_type(l, left)
+                and mypy.sametypes.is_same_type(r, right)):
             return True
     with pop_on_exit(assuming, left, right):
         for member in right.type.protocol_members:
@@ -483,7 +495,7 @@ def is_protocol_implementation(left: Instance, right: Instance,
                 is_compat = is_proper_subtype(subtype, supertype)
             if not is_compat:
                 return False
-            if isinstance(subtype, NoneTyp) and isinstance(supertype, CallableType):
+            if isinstance(subtype, NoneType) and isinstance(supertype, CallableType):
                 # We want __hash__ = None idiom to work even without --strict-optional
                 return False
             subflags = get_member_flags(member, left.type)
@@ -607,7 +619,10 @@ def find_node_type(node: Union[Var, FuncBase], itype: Instance, subtype: Type) -
     if typ is None:
         return AnyType(TypeOfAny.from_error)
     # We don't need to bind 'self' for static methods, since there is no 'self'.
-    if isinstance(node, FuncBase) or isinstance(typ, FunctionLike) and not node.is_staticmethod:
+    if (isinstance(node, FuncBase)
+            or (isinstance(typ, FunctionLike)
+                and node.is_initialized_in_class
+                and not node.is_staticmethod)):
         assert isinstance(typ, FunctionLike)
         signature = bind_self(typ, subtype)
         if node.is_property:
@@ -995,7 +1010,7 @@ def unify_generic_callable(type: CallableType, target: CallableType,
     return applied
 
 
-def restrict_subtype_away(t: Type, s: Type) -> Type:
+def restrict_subtype_away(t: Type, s: Type, *, ignore_promotions: bool = False) -> Type:
     """Return t minus s.
 
     If we can't determine a precise result, return a supertype of the
@@ -1012,8 +1027,10 @@ def restrict_subtype_away(t: Type, s: Type) -> Type:
         # TODO: Implement more robust support for runtime isinstance() checks,
         # see issue #3827
         new_items = [item for item in t.relevant_items()
-                     if (not (is_proper_subtype(erase_type(item), erased_s) or
-                              is_proper_subtype(item, erased_s))
+                     if (not (is_proper_subtype(erase_type(item), erased_s,
+                                                ignore_promotions=ignore_promotions) or
+                              is_proper_subtype(item, erased_s,
+                                                ignore_promotions=ignore_promotions))
                          or isinstance(item, AnyType))]
         return UnionType.make_union(new_items)
     else:
@@ -1056,9 +1073,9 @@ class ProperSubtypeVisitor(TypeVisitor[bool]):
     def visit_any(self, left: AnyType) -> bool:
         return isinstance(self.right, AnyType)
 
-    def visit_none_type(self, left: NoneTyp) -> bool:
-        if experiments.STRICT_OPTIONAL:
-            return (isinstance(self.right, NoneTyp) or
+    def visit_none_type(self, left: NoneType) -> bool:
+        if state.strict_optional:
+            return (isinstance(self.right, NoneType) or
                     is_named_instance(self.right, 'builtins.object'))
         return True
 
@@ -1091,7 +1108,7 @@ class ProperSubtypeVisitor(TypeVisitor[bool]):
                     elif variance == CONTRAVARIANT:
                         return self._is_proper_subtype(rightarg, leftarg)
                     else:
-                        return sametypes.is_same_type(leftarg, rightarg)
+                        return mypy.sametypes.is_same_type(leftarg, rightarg)
                 # Map left type to corresponding right instances.
                 left = map_instance_to_supertype(left, right.type)
 
@@ -1149,21 +1166,23 @@ class ProperSubtypeVisitor(TypeVisitor[bool]):
                     #       for isinstance(x, tuple), though it's unclear why.
                     return True
                 return all(self._is_proper_subtype(li, iter_type) for li in left.items)
-            return self._is_proper_subtype(left.fallback, right)
+            return self._is_proper_subtype(mypy.typeops.tuple_fallback(left), right)
         elif isinstance(right, TupleType):
             if len(left.items) != len(right.items):
                 return False
             for l, r in zip(left.items, right.items):
                 if not self._is_proper_subtype(l, r):
                     return False
-            return self._is_proper_subtype(left.fallback, right.fallback)
+            return self._is_proper_subtype(mypy.typeops.tuple_fallback(left),
+                                           mypy.typeops.tuple_fallback(right))
         return False
 
     def visit_typeddict_type(self, left: TypedDictType) -> bool:
         right = self.right
         if isinstance(right, TypedDictType):
             for name, typ in left.items.items():
-                if name in right.items and not is_same_type(typ, right.items[name]):
+                if (name in right.items
+                        and not mypy.sametypes.is_same_type(typ, right.items[name])):
                     return False
             for name, typ in right.items.items():
                 if name not in left.items:
@@ -1172,7 +1191,10 @@ class ProperSubtypeVisitor(TypeVisitor[bool]):
         return self._is_proper_subtype(left.fallback, right)
 
     def visit_literal_type(self, left: LiteralType) -> bool:
-        raise NotImplementedError()
+        if isinstance(self.right, LiteralType):
+            return left == self.right
+        else:
+            return self._is_proper_subtype(left.fallback, self.right)
 
     def visit_overloaded(self, left: Overloaded) -> bool:
         # TODO: What's the right thing to do here?
@@ -1186,7 +1208,6 @@ class ProperSubtypeVisitor(TypeVisitor[bool]):
         return False
 
     def visit_type_type(self, left: TypeType) -> bool:
-        # TODO: Handle metaclasses?
         right = self.right
         if isinstance(right, TypeType):
             # This is unsound, we don't check the __init__ signature.
@@ -1203,6 +1224,12 @@ class ProperSubtypeVisitor(TypeVisitor[bool]):
                 return True
             if right.type.fullname() == 'builtins.object':
                 return True
+            item = left.item
+            if isinstance(item, TypeVarType):
+                item = item.upper_bound
+            if isinstance(item, Instance):
+                metaclass = item.type.metaclass_type
+                return metaclass is not None and self._is_proper_subtype(metaclass, right)
         return False
 
 
