@@ -6,14 +6,17 @@ from mypy.test.helpers import Suite, assert_equal, assert_true, assert_false, as
 from mypy.erasetype import erase_type
 from mypy.expandtype import expand_type
 from mypy.join import join_types, join_simple
-from mypy.meet import meet_types
+from mypy.meet import meet_types, narrow_declared_type
+from mypy.sametypes import is_same_type
 from mypy.types import (
-    UnboundType, AnyType, CallableType, TupleType, TypeVarDef, Type, Instance, NoneTyp, Overloaded,
-    TypeType, UnionType, UninhabitedType, true_only, false_only, TypeVarId, TypeOfAny
+    UnboundType, AnyType, CallableType, TupleType, TypeVarDef, Type, Instance, NoneType,
+    Overloaded, TypeType, UnionType, UninhabitedType, true_only, false_only, TypeVarId, TypeOfAny,
+    LiteralType,
 )
 from mypy.nodes import ARG_POS, ARG_OPT, ARG_STAR, ARG_STAR2, CONTRAVARIANT, INVARIANT, COVARIANT
 from mypy.subtypes import is_subtype, is_more_precise, is_proper_subtype
 from mypy.test.typefixture import TypeFixture, InterfaceTypeFixture
+from mypy.state import strict_optional_set
 
 
 class TypesSuite(Suite):
@@ -41,7 +44,7 @@ class TypesSuite(Suite):
                          AnyType(TypeOfAny.special_form), self.function)
         assert_equal(str(c), 'def (X?, Y?) -> Any')
 
-        c2 = CallableType([], [], [], NoneTyp(), self.fx.function)
+        c2 = CallableType([], [], [], NoneType(), self.fx.function)
         assert_equal(str(c2), 'def ()')
 
     def test_callable_type_with_default_args(self) -> None:
@@ -85,7 +88,7 @@ class TypesSuite(Suite):
 
         v = [TypeVarDef('Y', 'Y', -1, [], self.fx.o),
              TypeVarDef('X', 'X', -2, [], self.fx.o)]
-        c2 = CallableType([], [], [], NoneTyp(), self.function, name=None, variables=v)
+        c2 = CallableType([], [], [], NoneType(), self.function, name=None, variables=v)
         assert_equal(str(c2), 'def [Y, X] ()')
 
 
@@ -245,6 +248,37 @@ class TypeOpsSuite(Suite):
         assert_false(is_proper_subtype(fx.gb, fx.ga))
         assert_false(is_proper_subtype(fx.ga, fx.gb))
 
+    def test_is_proper_subtype_and_subtype_literal_types(self) -> None:
+        fx = self.fx
+
+        lit1 = LiteralType(1, fx.a)
+        lit2 = LiteralType("foo", fx.d)
+        lit3 = LiteralType("bar", fx.d)
+
+        assert_true(is_proper_subtype(lit1, fx.a))
+        assert_false(is_proper_subtype(lit1, fx.d))
+        assert_false(is_proper_subtype(fx.a, lit1))
+        assert_true(is_proper_subtype(fx.uninhabited, lit1))
+        assert_false(is_proper_subtype(lit1, fx.uninhabited))
+        assert_true(is_proper_subtype(lit1, lit1))
+        assert_false(is_proper_subtype(lit1, lit2))
+        assert_false(is_proper_subtype(lit2, lit3))
+
+        assert_true(is_subtype(lit1, fx.a))
+        assert_false(is_subtype(lit1, fx.d))
+        assert_false(is_subtype(fx.a, lit1))
+        assert_true(is_subtype(fx.uninhabited, lit1))
+        assert_false(is_subtype(lit1, fx.uninhabited))
+        assert_true(is_subtype(lit1, lit1))
+        assert_false(is_subtype(lit1, lit2))
+        assert_false(is_subtype(lit2, lit3))
+
+        assert_false(is_proper_subtype(lit1, fx.anyt))
+        assert_false(is_proper_subtype(fx.anyt, lit1))
+
+        assert_true(is_subtype(lit1, fx.anyt))
+        assert_true(is_subtype(fx.anyt, lit1))
+
     # can_be_true / can_be_false
 
     def test_empty_tuple_always_false(self) -> None:
@@ -277,7 +311,7 @@ class TypeOpsSuite(Suite):
     # true_only / false_only
 
     def test_true_only_of_false_type_is_uninhabited(self) -> None:
-        to = true_only(NoneTyp())
+        to = true_only(NoneType())
         assert_type(UninhabitedType, to)
 
     def test_true_only_of_true_type_is_idempotent(self) -> None:
@@ -307,11 +341,20 @@ class TypeOpsSuite(Suite):
         assert_true(to.items[1] is tup_type)
 
     def test_false_only_of_true_type_is_uninhabited(self) -> None:
-        fo = false_only(self.tuple(AnyType(TypeOfAny.special_form)))
-        assert_type(UninhabitedType, fo)
+        with strict_optional_set(True):
+            fo = false_only(self.tuple(AnyType(TypeOfAny.special_form)))
+            assert_type(UninhabitedType, fo)
+
+    def test_false_only_tuple(self) -> None:
+        with strict_optional_set(False):
+            fo = false_only(self.tuple(self.fx.a))
+            assert_equal(fo, NoneType())
+        with strict_optional_set(True):
+            fo = false_only(self.tuple(self.fx.a))
+            assert_equal(fo, UninhabitedType())
 
     def test_false_only_of_false_type_is_idempotent(self) -> None:
-        always_false = NoneTyp()
+        always_false = NoneType()
         fo = false_only(always_false)
         assert_true(always_false is fo)
 
@@ -325,18 +368,19 @@ class TypeOpsSuite(Suite):
         assert_true(self.fx.a.can_be_true)
 
     def test_false_only_of_union(self) -> None:
-        tup_type = self.tuple()
-        # Union of something that is unknown, something that is always true, something
-        # that is always false
-        union_type = UnionType([self.fx.a, self.tuple(AnyType(TypeOfAny.special_form)),
-                                tup_type])
-        assert_equal(len(union_type.items), 3)
-        fo = false_only(union_type)
-        assert isinstance(fo, UnionType)
-        assert_equal(len(fo.items), 2)
-        assert_false(fo.items[0].can_be_true)
-        assert_true(fo.items[0].can_be_false)
-        assert_true(fo.items[1] is tup_type)
+        with strict_optional_set(True):
+            tup_type = self.tuple()
+            # Union of something that is unknown, something that is always true, something
+            # that is always false
+            union_type = UnionType([self.fx.a, self.tuple(AnyType(TypeOfAny.special_form)),
+                                    tup_type])
+            assert_equal(len(union_type.items), 3)
+            fo = false_only(union_type)
+            assert isinstance(fo, UnionType)
+            assert_equal(len(fo.items), 2)
+            assert_false(fo.items[0].can_be_true)
+            assert_true(fo.items[0].can_be_false)
+            assert_true(fo.items[1] is tup_type)
 
     # Helpers
 
@@ -418,10 +462,10 @@ class JoinSuite(Suite):
 
     def test_none(self) -> None:
         # Any type t joined with None results in t.
-        for t in [NoneTyp(), self.fx.a, self.fx.o, UnboundType('x'),
+        for t in [NoneType(), self.fx.a, self.fx.o, UnboundType('x'),
                   self.fx.t, self.tuple(),
                   self.callable(self.fx.a, self.fx.b), self.fx.anyt]:
-            self.assert_join(t, NoneTyp(), t)
+            self.assert_join(t, NoneType(), t)
 
     def test_unbound_type(self) -> None:
         self.assert_join(UnboundType('x'), UnboundType('x'), self.fx.anyt)
@@ -436,7 +480,7 @@ class JoinSuite(Suite):
 
     def test_any_type(self) -> None:
         # Join against 'Any' type always results in 'Any'.
-        for t in [self.fx.anyt, self.fx.a, self.fx.o, NoneTyp(),
+        for t in [self.fx.anyt, self.fx.a, self.fx.o, NoneType(),
                   UnboundType('x'), self.fx.t, self.tuple(),
                   self.callable(self.fx.a, self.fx.b)]:
             self.assert_join(t, self.fx.anyt, self.fx.anyt)
@@ -590,6 +634,41 @@ class JoinSuite(Suite):
         self.assert_join(self.fx.type_type, self.fx.type_any, self.fx.type_type)
         self.assert_join(self.fx.type_b, self.fx.anyt, self.fx.anyt)
 
+    def test_literal_type(self) -> None:
+        a = self.fx.a
+        d = self.fx.d
+        lit1 = LiteralType(1, a)
+        lit2 = LiteralType(2, a)
+        lit3 = LiteralType("foo", d)
+
+        self.assert_join(lit1, lit1, lit1)
+        self.assert_join(lit1, a, a)
+        self.assert_join(lit1, d, self.fx.o)
+        self.assert_join(lit1, lit2, a)
+        self.assert_join(lit1, lit3, self.fx.o)
+        self.assert_join(lit1, self.fx.anyt, self.fx.anyt)
+        self.assert_join(UnionType([lit1, lit2]), lit2, UnionType([lit1, lit2]))
+        self.assert_join(UnionType([lit1, lit2]), a, a)
+        self.assert_join(UnionType([lit1, lit3]), a, UnionType([a, lit3]))
+        self.assert_join(UnionType([d, lit3]), lit3, UnionType([d, lit3]))
+        self.assert_join(UnionType([d, lit3]), d, UnionType([d, lit3]))
+        self.assert_join(UnionType([a, lit1]), lit1, UnionType([a, lit1]))
+        self.assert_join(UnionType([a, lit1]), lit2, UnionType([a, lit1]))
+        self.assert_join(UnionType([lit1, lit2]),
+                         UnionType([lit1, lit2]),
+                         UnionType([lit1, lit2]))
+
+        # The order in which we try joining two unions influences the
+        # ordering of the items in the final produced unions. So, we
+        # manually call 'assert_simple_join' and tune the output
+        # after swapping the arguments here.
+        self.assert_simple_join(UnionType([lit1, lit2]),
+                                UnionType([lit2, lit3]),
+                                UnionType([lit1, lit2, lit3]))
+        self.assert_simple_join(UnionType([lit2, lit3]),
+                                UnionType([lit1, lit2]),
+                                UnionType([lit2, lit3, lit1]))
+
     # There are additional test cases in check-inference.test.
 
     # TODO: Function types + varargs and default args.
@@ -642,8 +721,8 @@ class MeetSuite(Suite):
         self.assert_meet(self.fx.a, self.fx.o, self.fx.a)
         self.assert_meet(self.fx.a, self.fx.b, self.fx.b)
         self.assert_meet(self.fx.b, self.fx.o, self.fx.b)
-        self.assert_meet(self.fx.a, self.fx.d, NoneTyp())
-        self.assert_meet(self.fx.b, self.fx.c, NoneTyp())
+        self.assert_meet(self.fx.a, self.fx.d, NoneType())
+        self.assert_meet(self.fx.b, self.fx.c, NoneType())
 
     def test_tuples(self) -> None:
         self.assert_meet(self.tuple(), self.tuple(), self.tuple())
@@ -652,14 +731,14 @@ class MeetSuite(Suite):
                          self.tuple(self.fx.a))
         self.assert_meet(self.tuple(self.fx.b, self.fx.c),
                          self.tuple(self.fx.a, self.fx.d),
-                         self.tuple(self.fx.b, NoneTyp()))
+                         self.tuple(self.fx.b, NoneType()))
 
         self.assert_meet(self.tuple(self.fx.a, self.fx.a),
                          self.fx.std_tuple,
                          self.tuple(self.fx.a, self.fx.a))
         self.assert_meet(self.tuple(self.fx.a),
                          self.tuple(self.fx.a, self.fx.a),
-                         NoneTyp())
+                         NoneType())
 
     def test_function_types(self) -> None:
         self.assert_meet(self.callable(self.fx.a, self.fx.b),
@@ -676,17 +755,17 @@ class MeetSuite(Suite):
     def test_type_vars(self) -> None:
         self.assert_meet(self.fx.t, self.fx.t, self.fx.t)
         self.assert_meet(self.fx.s, self.fx.s, self.fx.s)
-        self.assert_meet(self.fx.t, self.fx.s, NoneTyp())
+        self.assert_meet(self.fx.t, self.fx.s, NoneType())
 
     def test_none(self) -> None:
-        self.assert_meet(NoneTyp(), NoneTyp(), NoneTyp())
+        self.assert_meet(NoneType(), NoneType(), NoneType())
 
-        self.assert_meet(NoneTyp(), self.fx.anyt, NoneTyp())
+        self.assert_meet(NoneType(), self.fx.anyt, NoneType())
 
         # Any type t joined with None results in None, unless t is Any.
         for t in [self.fx.a, self.fx.o, UnboundType('x'), self.fx.t,
                   self.tuple(), self.callable(self.fx.a, self.fx.b)]:
-            self.assert_meet(t, NoneTyp(), NoneTyp())
+            self.assert_meet(t, NoneType(), NoneType())
 
     def test_unbound_type(self) -> None:
         self.assert_meet(UnboundType('x'), UnboundType('x'), self.fx.anyt)
@@ -703,7 +782,7 @@ class MeetSuite(Suite):
 
     def test_dynamic_type(self) -> None:
         # Meet against dynamic type always results in dynamic.
-        for t in [self.fx.anyt, self.fx.a, self.fx.o, NoneTyp(),
+        for t in [self.fx.anyt, self.fx.a, self.fx.o, NoneType(),
                   UnboundType('x'), self.fx.t, self.tuple(),
                   self.callable(self.fx.a, self.fx.b)]:
             self.assert_meet(t, self.fx.anyt, t)
@@ -780,8 +859,34 @@ class MeetSuite(Suite):
         self.assert_meet(self.fx.type_type, self.fx.type_any, self.fx.type_any)
         self.assert_meet(self.fx.type_b, self.fx.anyt, self.fx.type_b)
 
+    def test_literal_type(self) -> None:
+        a = self.fx.a
+        d = self.fx.d
+        lit1 = LiteralType(1, a)
+        lit2 = LiteralType(2, a)
+        lit3 = LiteralType("foo", d)
+
+        self.assert_meet(lit1, lit1, lit1)
+        self.assert_meet(lit1, a, lit1)
+        self.assert_meet_uninhabited(lit1, lit3)
+        self.assert_meet_uninhabited(lit1, lit2)
+        self.assert_meet(UnionType([lit1, lit2]), lit1, lit1)
+        self.assert_meet(UnionType([lit1, lit2]), UnionType([lit2, lit3]), lit2)
+        self.assert_meet(UnionType([lit1, lit2]), UnionType([lit1, lit2]), UnionType([lit1, lit2]))
+        self.assert_meet(lit1, self.fx.anyt, lit1)
+        self.assert_meet(lit1, self.fx.o, lit1)
+
+        assert_true(is_same_type(lit1, narrow_declared_type(lit1, a)))
+        assert_true(is_same_type(lit2, narrow_declared_type(lit2, a)))
+
     # FIX generic interfaces + ranges
 
+    def assert_meet_uninhabited(self, s: Type, t: Type) -> None:
+        with strict_optional_set(False):
+            self.assert_meet(s, t, self.fx.nonet)
+        with strict_optional_set(True):
+            self.assert_meet(s, t, self.fx.uninhabited)
+
     def assert_meet(self, s: Type, t: Type, meet: Type) -> None:
         self.assert_simple_meet(s, t, meet)
         self.assert_simple_meet(t, s, meet)
@@ -808,3 +913,46 @@ class MeetSuite(Suite):
         return CallableType(list(a[:-1]),
                             [ARG_POS] * n, [None] * n,
                             a[-1], self.fx.function)
+
+
+class SameTypeSuite(Suite):
+    def setUp(self) -> None:
+        self.fx = TypeFixture()
+
+    def test_literal_type(self) -> None:
+        b = self.fx.b  # Reminder: b is a subclass of a
+        d = self.fx.d
+
+        lit1 = LiteralType(1, b)
+        lit2 = LiteralType(2, b)
+        lit3 = LiteralType("foo", d)
+
+        self.assert_same(lit1, lit1)
+        self.assert_same(UnionType([lit1, lit2]), UnionType([lit1, lit2]))
+        self.assert_same(UnionType([lit1, lit2]), UnionType([lit2, lit1]))
+        self.assert_not_same(lit1, b)
+        self.assert_not_same(lit1, lit2)
+        self.assert_not_same(lit1, lit3)
+
+        self.assert_not_same(lit1, self.fx.anyt)
+        self.assert_not_same(lit1, self.fx.nonet)
+
+    def assert_same(self, s: Type, t: Type, strict: bool = True) -> None:
+        self.assert_simple_is_same(s, t, expected=True, strict=strict)
+        self.assert_simple_is_same(t, s, expected=True, strict=strict)
+
+    def assert_not_same(self, s: Type, t: Type, strict: bool = True) -> None:
+        self.assert_simple_is_same(s, t, False, strict=strict)
+        self.assert_simple_is_same(t, s, False, strict=strict)
+
+    def assert_simple_is_same(self, s: Type, t: Type, expected: bool, strict: bool) -> None:
+        actual = is_same_type(s, t)
+        assert_equal(actual, expected,
+                     'is_same_type({}, {}) is {{}} ({{}} expected)'.format(s, t))
+
+        if strict:
+            actual2 = (s == t)
+            assert_equal(actual2, expected,
+                         '({} == {}) is {{}} ({{}} expected)'.format(s, t))
+            assert_equal(hash(s) == hash(t), expected,
+                         '(hash({}) == hash({}) is {{}} ({{}} expected)'.format(s, t))
